\name{anova.gam}
\alias{anova.gam}
\alias{summary.gam}
\title{Analysis of Deviance for a Generalized Additive Model}
\description{Produces an ANODEV table for a set of GAM models, or else a
summary for a single GAM model}
\usage{
\method{anova}{gam}(object, \dots, test)
\method{summary}{gam}(object, dispersion=NULL,\dots)
}
\arguments{
  \item{object}{a fitted gam}
  \item{\dots}{other fitted gams for \code{anova}}
  \item{test}{a character string specifying the test statistic to be used.
          Can be one of '"F"', '"Chisq"' or '"Cp"', with partial
          matching allowed, or 'NULL' for no test.}
	\item{dispersion}{a dispersion parameter to be used in computing
	  standard errors}
	}
\details{
These are  methods for the functions \code{anova} or \code{summary} for objects inheriting from class `gam'.
See \code{\link{anova}} for the general behavior of this function
and for the interpretation of `test'.


When called with a single `gam' object, a special anova table for `gam'
models is returned. This gives a breakdown of the degrees of freedom for
all the terms in the model, separating the projection part and
nonparametric part of each. For example, a term specified by `s()' is
broken down into a single degree of freedom for its linear component,
and the remainder for the nonparametric component. In addition, a type
of score test is performed for each of the nonparametric terms. The
nonparametric component is set to zero, and the linear part is updated,
holding the other nonparametric terms fixed. This is done efficiently
and simulataneously for all terms.  }
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992). 
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.

  Venables, W. N. and Ripley, B. D. (2002)
  \emph{Modern Applied Statistics with S.}
  New York: Springer.
}

\examples{
data(gam.data)
gam.object <- gam(y~s(x,6)+z,data=gam.data)
anova(gam.object)
gam.object2 <- update(gam.object, ~.-z)
anova(gam.object, gam.object2, test="Chisq")
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
\name{gam-internal}
\title{Service functions and as yet undocumented functions for the gam library}
\alias{.First.lib}
\alias{[.smooth}
\alias{all.wam}
\alias{anova.gamlist}
\alias{as.anova}
\alias{as.data.frame.lo.smooth}
\alias{assign.list}
\alias{deviance.default}
\alias{deviance.glm}
\alias{deviance.lm}
\alias{gamlist}
\alias{gam.match}
\alias{gam.nlchisq}
\alias{gam.random}
\alias{gam.scope}
\alias{gam.slist}
\alias{gam.sp}
\alias{gam.wlist}
\alias{gplot}
\alias{gplot.default}
\alias{gplot.factor}
\alias{gplot.list}
\alias{gplot.matrix}
\alias{gplot.numeric}
\alias{labels.gam}
\alias{newdata.predict.gam}
\alias{polylo}
\alias{print.gam}
\alias{print.summary.gam}
\alias{random}
\alias{ylim.scale}
\description{Internal gam functions}
\author{Trevor Hastie}
\keyword{internal}

\eof
\name{gam}
\title{Fitting Generalized Additive Models}
\alias{gam}
\alias{gam.fit}
\concept{nonparametric}
\concept{additive}
\concept{regression}
\concept{logistic}
\concept{log-linear}
\concept{loglinear}
\description{
  \code{gam} is used to fit generalized additive models, specified by
  giving a symbolic description of the additive predictor and a
  description of the error distribution. \code{gam} uses the
  \emph{backfitting algorithm} to combine different smoothing or
  fitting methods. The methods currently supported are local regression
  and smoothing splines.
  }
\usage{
gam(formula, family = gaussian, data, weights, subset, na.action, 
       start, etastart, mustart, control = gam.control(\ldots),
model=FALSE, method, x=FALSE, y=TRUE, \dots)

gam.fit(x, y, smooth.frame, weights = rep(1,nobs), start = NULL, 
    etastart = NULL, mustart = NULL, offset = rep(0, nobs), family = gaussian(), 
    control = gam.control()) 
}
\arguments{

  \item{formula}{a formula expression as for other regression models, of
    the form \code{response ~ predictors}. See the documentation of
    \code{lm} and \code{formula} for details.  Built-in nonparametric
smoothing terms are indicated by \code{s} for smoothing splines or
\code{lo} for \code{loess} smooth terms.  See the documentation for
\code{s} and \code{lo} for their arguments. Additional smoothers can be
added by creating the appropriate interface functions. Interactions with
nonparametric smooth terms are not fully supported, but will not produce
errors; they will simply produce the usual parametric interaction.}

\item{family}{a description of the error distribution and link
    function to be used in the model. This can be a character string
    naming a family function, a family function or the result of a call
    to a family function.  (See \code{\link{family}} for details of
    family functions.)}
  
  \item{data}{an optional data frame containing the variables
    in the model.  If not found in \code{data}, the variables are taken
    from \code{environment(formula)}, typically the environment from
    which \code{gam} is called.}

  \item{weights}{an optional vector of weights to be used
    in the fitting process.}

  \item{subset}{an optional vector specifying a subset of observations
    to be used in the fitting process.}

  \item{na.action}{a function which indicates what should happen
    when the data contain \code{NA}s.  The default is set by
    the \code{na.action} setting of \code{\link{options}}, and is
    \code{\link{na.fail}} if that is unset.  The \dQuote{factory-fresh}
    default is \code{\link{na.omit}}. A special method
    \code{\link{na.gam.replace}} allows for mean-imputation of missing
    values (assumes missing at random), and works gracefully with \code{gam}}

  \item{start}{starting values for the parameters in the additive predictor.}

  \item{etastart}{starting values for the additive predictor.}

  \item{mustart}{starting values for the vector of means.}

  \item{offset}{this can be used to specify an \emph{a priori}
    known component to be included in the additive predictor
    during fitting.}

  \item{control}{a list of parameters for controlling the fitting
    process.  See the documentation for \code{\link{gam.control}}
    for details. These can also be set as arguments to \code{gam()} itself.
}

  \item{model}{a logical value indicating whether \emph{model frame}
    should be included as a component of the returned value.}

  \item{method}{the method to be used in fitting the parametric part of
    the model.
    The default method \code{"glm.fit"} uses iteratively reweighted
    least squares (IWLS).  The only current alternative is
    \code{"model.frame"} which returns the model frame and does no fitting.}

  \item{x, y}{For \code{gam}:
    logical values indicating whether the response
    vector and model matrix used in the fitting process
    should be returned as components of the returned value.

    For \code{gam.fit}: \code{x} is a model matrix of dimension \code{n
      * p}, and \code{y} is a vector of observations of length \code{n}.
    }
\item{smooth.frame}{for \code{gam.fit} only. This is essentially a
  subset of the model frame corresponding to the smooth terms, and has
  the ingredients needed for smoothing each variable in the backfitting
  algorithm. The elements of this frame are produced by the formula
  functions \code{lo} and \code{s}.}
    \item{\dots}{further arguments passed to or from other methods.}
  }

\details{
 The gam model is fit using the local scoring algorithm, which
iteratively fits weighted additive models by backfitting. The
backfitting algorithm is a Gauss-Seidel method for fitting additive
models, by iteratively smoothing partial residuals.  The algorithm
separates the parametric from the nonparametric part of the fit, and
fits the parametric part using weighted linear least squares within the
backfitting algorithm. This version of \code{gam} remains faithful to
the philosophy of GAM models as outlined in the references below.

An object \code{gam.slist} (currently set to
\code{c("lo","s","random")}) lists the smoothers supported by
\code{gam}. Corresponding to each of these is a smoothing function
\code{gam.lo}, \code{gam.s} etc that take particular arguments and
produce particular output, custom built to serve as building blocks in
the backfitting algorithm. This allows users to add their own smoothing
methods. See the documentation for these methods for further information.
}
\value{
\code{gam} returns an object of class \code{gam}, which inherits from
both \code{glm} and \code{lm}.

Gam objects can be examined by \code{print}, \code{summary},
\code{plot}, and \code{anova}.  Components can be extracted using
extractor functions \code{predict}, \code{fitted}, \code{residuals},
\code{deviance}, \code{formula}, and \code{family}. Can be modified
using \code{update}. It has all the components of a \code{glm} object,
with a few more. This also means it can be queried, summarized etc by
methods for \code{glm} and \code{lm} objects. Other generic functions
that have methods for \code{gam} objects are \code{step} and
\code{preplot}.

The following components must be included in a legitimate `gam' object.
The residuals, fitted values,  coefficients and effects should be extracted
by the generic functions of the same name, rather than
by the \code{`$'} operator. 
The \code{family} function returns the entire family object used in the fitting, and \code{deviance} can be used to extract the deviance of the fit. 

\item{coefficients}{
the coefficients of the parametric part of the \code{additive.predictors}, which multiply  the
columns of the model
matrix.
The names of the coefficients are the names of the
single-degree-of-freedom effects (the columns of the
model matrix).
If the model is overdetermined there will
be missing values in the coefficients corresponding to inestimable
coefficients.
}
\item{additive.predictors}{
the additive fit, given by the product of the model matrix and the coefficients, plus the columns of the \code{$smooth} component.
}
\item{fitted.values}{
the fitted mean values, obtained by transforming  the component \code{additive.predictors} using the inverse link function.
}
\item{smooth, nl.df, nl.chisq, var}{
these four characterize the nonparametric aspect of the fit.
\code{smooth} is a matrix of smooth terms, with a column corresponding to each smooth term in the model; if no smooth terms are in the \code{gam} model, all these components will be missing. 
Each column corresponds to the strictly  nonparametric part of the term, while the parametric part is obtained from the model matrix.
\code{nl.df} is a vector giving the approximate degrees of freedom for each column of \code{smooth}. For smoothing splines specified by \code{s(x)}, the approximate \code{df} will be the trace of the implicit smoother matrix minus 2.
\code{nl.chisq} is a vector containing a type of score test for the removal of each of the columns of \code{smooth}.
\code{var} is a matrix like \code{smooth}, containing the approximate pointwise variances for the columns of \code{smooth}.
}
\item{smooth.frame}{This is essentially a subset of the model frame
  corresponding to the smooth terms, and has the ingredients needed for
  making predictions from a \code{gam} object}
\item{residuals}{
the  residuals from the final weighted additive fit; also known as 
residuals, these are typically not interpretable without rescaling by the weights.
}
\item{deviance}{
up to a constant, minus twice the maximized log-likelihood. Similar to
the residual sum of squares. Where sensible, the constant is chosen so that a
    saturated model has deviance zero.
}
\item{null.deviance}{The deviance for the null model, comparable with
    \code{deviance}. The null model will include the offset, and an
    intercept if there is one in the model}
\item{iter}{
the number of local scoring 
iterations used to compute the estimates. 
}
\item{family}{
a three-element character vector giving the name of the family, the link, and the variance function; mainly for printing purposes.
}
\item{weights}{the \emph{working} weights, that is the weights
    in the final iteration of the local scoring fit.}
\item{prior.weights}{the case weights initially supplied.}
\item{df.residual}{the residual degrees of freedom.}
\item{df.null}{the residual degrees of freedom for the null model.}

The object will also have the components of a \code{lm} object:
\code{coefficients}, \code{residuals}, \code{fitted.values},
\code{call}, \code{terms},  and some
others involving the numerical fit.  See \code{lm.object}.

}


\seealso{
\code{\link{glm}},  \code{\link{family}}, \code{\link{lm}}.
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992), and the philosophy in Hastie and Tibshirani (1991).
  This version of \code{gam} is adapted from the S
  version to match the \code{glm} and \code{lm} functions in R. 

  Note that this version of \code{gam} is different from the function
with
  the same name in the R library \code{mgcv}, which uses only smoothing
  splines with a focus on automatic smoothing parameter selection via
  GCV.

 }
\references{
  Hastie, T. J. (1990)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.

  Venables, W. N. and Ripley, B. D. (2002)
  \emph{Modern Applied Statistics with S.}
  New York: Springer.
}

\examples{
data(kyphosis)
gam(Kyphosis ~ s(Age,4) + Number, family = binomial, data=kyphosis,
trace=TRUE)
data(airquality)
gam(Ozone^(1/3) ~ lo(Solar.R) + lo(Wind, Temp), data=airquality, na=na.gam.replace)
gam(Kyphosis ~ poly(Age,2) + s(Start), data=kyphosis, family=binomial, subset=Number>2)
data(gam.data)
gam.object <- gam(y ~ s(x,6) + z,data=gam.data)
summary(gam.object)
plot(gam.object,se=TRUE)
data(gam.newdata)
predict(gam.object,type="terms",newdata=gam.newdata)
}

\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}

\eof
\name{gam.control}
\alias{gam.control}
\title{Auxilliary for controlling GAM fitting}
\description{Auxiliary function as user interface for 'gam' fitting. Typically
     only used when calling 'gam' or 'gam.fit'.}
\usage{
gam.control(epsilon=1e-07, bf.epsilon = 1e-07, maxit=30, bf.maxit = 30, trace=FALSE)
}
\arguments{
\item{epsilon}{
convergence threshold for local scoring iterations
}
\item{bf.epsilon}{
convergence threshold for backfitting iterations
}
\item{maxit}{
maximum number of local scoring iterations
}
\item{bf.maxit}{
maximum number of backfitting iterations
}
\item{trace}{
should iteration details be printed while \code{gam} is fitting the model.
}}
\value{
a list is returned, consisting of the five parameters, conveniently packaged up to supply the \code{control} argument to \code{gam}. The values for \code{gam.control} can be supplied directly in a call to \code{gam}; these are then filtered through \code{gam.control} inside \code{gam}.
}
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.
  }
\examples{
\dontrun{gam(formula, family, control = gam.control(bf.maxit=15))}
\dontrun{gam(formula, family, bf.maxit = 15) # these are equivalent}
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}

\eof
\name{gam.data}
\alias{gam.data}
\alias{gam.newdata}
\docType{data}
\title{Simulated dataset for gam}
\description{
A simple simulated dataset, used to test out the gam functions
}
\usage{
data(gam.data)
data(gam.newdata)
}
\format{
  A data frame with 100 observations on the following 6 variables.
  \describe{
    \item{x}{a numeric vector - predictor}
    \item{y}{a numeric vector - the response}
    \item{z}{a numeric vector - noise predictor}
    \item{f}{a numeric vector - true function}
    \item{probf}{a numeric vector - probability function}
    \item{ybin}{a numeric vector - binary response}
  }
}
\details{
This dataset is artificial, and is used to test out some of the features
of gam.
}
\examples{
data(gam.data)
gam(y ~ s(x) + z, data=gam.data)
}
\keyword{datasets}

\eof
\name{kyphosis}
\alias{kyphosis}
\docType{data}
\title{A classic example dataset for GAMs}
\description{
Data on the results of a spinal operation "laminectomy" on children, to
correct for a condition called "kyphosis"; see Hastie and
Tibshirani (1990) for details}
}
\usage{
data(kyphosis)
}
\format{A data frame with 81 observations on the following 4 variables.
  \describe{
    \item{Kyphosis}{a response factor with levels \code{absent}
      \code{present}.}
    \item{Age}{of child in months, a numeric vector}
    \item{Number}{of vertebra involved in the operation,a numeric vector}
    \item{Start}{level of the operation, a numeric vector}
  }
}
}
\source{
  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.
}
\keyword{datasets}

\eof
\name{lo}
\alias{lo}
\alias{gam.lo}
\title{Specify a loess fit in a GAM formula}
\description{A symbolic wrapper to indicate a smooth term in a formala
 argument to gam}
\usage{
lo(\dots, span=0.5, degree=1)
gam.lo(x, y, w, span, degree, ncols, xeval)
}
\arguments{
\item{...}{
the unspecified \code{\dots}  can be a comma-separated list of numeric vectors, numeric matrix, or expressions that evaluate to either of these. If it is a list of vectors, they must all have the same length.}
\item{span}{
the number of observations in a neighborhood. This is the smoothing parameter for a \code{loess} fit.}
\item{degree}{the degree of local polynomial to be fit; currently restricted to be \code{1} or \code{2}.}
\item{x}{for \code{gam.lo}, the appropriate basis of polynomials
  generated from the arguments to \code{lo}. These are also the
  variables that receive linear coefficients in the GAM fit.}
\item{y}{a response variable passed to \code{gam.lo} during backfitting}
\item{w}{weights}
\item{ncols}{for \code{gam.lo} the number of columns in \code{x} used as
  the smoothing inputs to local regression. For example, if
  \code{degree=2}, then \code{x} has two columns defining a degree-2
  polynomial basis. Both are needed for the parameteric part of the fit,
  but \code{ncol=1} telling the local regression routine that the first
  column is the actually smoothing variable.}
\item{xeval}{If this argument is present, then \code{gam.lo} produces a
  prediction at \code{xeval}.}
}
\value{
\code{lo} returns a numeric matrix.  The simplest case is when there is a
single argument to \code{lo} and \code{degree=1}; a one-column matrix is
returned, consisting of a normalized version of the vector.  If
\code{degree=2} in this case, a two-column matrix is returned, consisting
of a degree-2 polynomial basis.  Similarly, if there are
two arguments, or the single argument is a two-column matrix, either a
two-column matrix is returned if \code{degree=1}, or a five-column matrix
consisting of powers and products up to degree \code{2}.  Any dimensional
argument is allowed, but typically one or two vectors are used in
practice.

The matrix is endowed with a number of attributes; the matrix itself is
used in the construction of the model matrix, while the attributes are
needed for the backfitting algorithms \code{all.wam} (weighted additive
model) or \code{lo.wam} (currently not implemented). Local-linear curve
or surface fits reproduce linear responses, while local-quadratic fits
reproduce quadratic curves or surfaces. These parts of the \code{loess}
fit are computed exactly together with the other parametric linear parts


Note that \code{lo} itself does no smoothing; it simply sets things up
for \code{gam}; \code{gam.lo} does the actual smoothing.
of the model.

One important attribute is named \code{call}. For example, \code{lo(x)}
has a call component
\code{gam.lo(data[["lo(x)"]], z, w, span = 0.5, degree = 1, ncols = 1)}.
This is an expression that gets evaluated repeatedly in \code{all.wam}
(the backfitting algorithm).

\code{gam.lo} returns an object with components
\item{residuals}{The residuals from the smooth fit. Note that the
  smoother removes the parametric part of the fit (using a linear fit
  with the columns in \code{x}), so these residual represent the
  nonlinear part of the fit.}
\item{nl.df}{the nonlinear degrees of freedom}
\item{var}{the pointwise variance for the nonlinear fit}

When \code{gam.lo} is evaluated with an \code{xeval} argument, it returns a
matrix of predictions.
}
\details{
  A smoother in gam separates out the parametric part of the fit
  from the non-parametric part. For local regression, the parametric
  part of the fit is specified by the particular polynomial being fit
  locally. The workhorse function \code{gam.lo} fits the local
  polynomial, then strips off this parametric part. All the parametric
  pieces from all the terms in the additive model are fit
  simultaneously in one operation for each loop of the backfitting
  algorithm.
  }
\seealso{
\code{\link{s}}, \code{\link{bs}}, \code{\link{ns}}, \code{\link{poly}}, \code{\link{loess}} 
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992).
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.
}
\examples{
y ~ Age + lo(Start, span=.5)
     # fit Start using a loess smooth with a span of 0.5.
y ~ lo(Age) + lo(Start, Number) 
y ~ lo(Age, 0.5) # the argument name for span is not needed.
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
\name{na.gam.replace}
\alias{na.gam.replace}
\title{Missing Data Filter for GAMs}
\description{A method for dealing with missing values, friendly to GAM models.}
\usage{
na.gam.replace(frame)
}
\arguments{
\item{frame}{
a model or data frame
}}
\value{
a model or data frame is returned, with the missing observations (NAs) replaced.
The following rules are used. A factor with missing data is replaced by a new factor with one more level, labelled \code{"NA"}, which records the missing data. 
Ordered factors are treated similarly, except the result is an unordered factor.
A missing numeric vector has its missing entires replaced by the mean of the non-missing entries. Similarly, a matrix with missing entries has each missing entry replace by the mean of its column.
If \code{frame} is a model frame, the response variable can be identified, as can the weights (if present). Any rows for which the response or weight is missing are removed entirely from the model frame.


The word \code{"gam"} in the name is relevant, because \code{gam()} makes special use of this filter. All columns of a model frame that were created by a call to \code{lo()} or \code{s()} have an attribute names \code{"NAs"} if NAs are present in their columns. 
Despite the replacement by means, these attributes remain on the object,
and \code{gam()} takes appropriate action when smoothing against these
columns. See section 7.3.2 in Hastie (1992)
for more details.
}
\author{Trevor Hastie}

\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.
}

\examples{
data(airquality)
gam(Ozone^(1/3) ~ lo(Solar.R) + lo(Wind, Temp), data=airquality, na=na.gam.replace)
}
\seealso{
\code{\link{na.fail}}, \code{\link{na.omit}}, \code{\link{gam}}
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
\name{plot.gam}
\alias{plot.gam}
\alias{preplot.gam}
\alias{plot.preplot.gam}
\title{Plot Components of a GAM Object}
\description{A plot method for GAM objects, which can be used on GLM and
  LM
  objects as well. It focuses on terms (main-effects), and produces a
  suitable plot for terms of different types}
\usage{
plot.gam(x, residuals, rugplot, se, scale, ask = FALSE,\dots)
preplot.gam(object, newdata, terms,\dots)
}
\arguments{
\item{x}{
a \code{gam} object, or a \code{preplot.gam} object. The first thing \code{plot.gam()} does is check if \code{x} has a component called \code{preplot}; if not, it computes one using \code{preplot.gam()}. Either way, it is this \code{preplot.gam} object that is required for plotting a \code{gam} object.
}
\item{object}{same as \code{x}}
\item{residuals}{
if \code{TRUE}, partial deviance residuals are plotted along with the fitted terms---default is \code{FALSE}. If \code{residuals} is a vector with the same length as each fitted term in \code{x}, then these are taken to be the overall residuals to be used for constructing the partial residuals.
}
\item{rugplot}{
if \code{TRUE} (the default), a univariate histogram or \code{rugplot} is displayed along the base of each plot, showing the occurrence of each `x'; ties are broken by jittering.
}
\item{se}{
if \code{TRUE}, upper and lower pointwise twice-standard-error curves are included for each plot. The default is \code{FALSE}.
}
\item{scale}{
a lower limit for the number of units covered by the limits on the `y' for each plot. The default is \code{scale=0}, in which case each plot uses the range of the functions being plotted to create their \code{ylim}. By setting \code{scale} to be the maximum value of \code{diff(ylim)} for all the plots, then all subsequent plots will produced in the same vertical units. This is essential for comparing the importance of fitted terms in additive models.
}
\item{ask}{
if \code{TRUE}, \code{plot.gam()} operates in interactive mode. 
}
\item{newdata}{if supplied to \code{preplot.gam}, the preplot object is
  based on them rather than the original.}
\item{terms}{subsets of the terms can be selected}
\item{\dots}{}
}
\value{
a plot is produced for each of the terms in the object \code{x}. The function currently knows how to plot all main-effect functions of one or two predictors. So in particular, interactions are not plotted. An appropriate `x-y' is produced to display each of the terms, adorned with residuals, standard-error curves, and a rugplot, depending on the choice of options. The form of the plot is different, depending on whether the `x'-value for each plot is numeric, a factor, or a matrix.


When \code{ask=TRUE}, rather than produce each plot sequentially, \code{plot.gam()} displays a menu listing all the terms that can be plotted, as well as switches for all the options. 


A \code{preplot.gam} object is a list of precomputed terms. Each such
term (also a \code{preplot.gam} object) is a list with components
\code{x}, \code{y} and others---the basic ingredients needed for each
term plot. These are in turn handed to the specialized plotting function
\code{gplot()}, which has methods for different classes of the leading
\code{x} argument. In particular, a different plot is produced if
\code{x} is numeric, a category or factor, a matrix, or a
list. Experienced users can extend this range by creating more
\code{gplot()} methods for other classes.  Graphical parameters (see
\code{\link{par}}) may also be supplied as arguments to this function.
This function is a method for the generic function \code{plot()} for
class \code{"gam"}.

It can be invoked by calling \code{plot(x)} for an
object \code{x} of the appropriate class, or directly by
calling \code{plot.gam(x)} regardless of the
class of the object.
}
\seealso{
\code{\link{preplot}}, \code{\link{predict.gam}}
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992).
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.
}
\examples{
data(gam.data)
gam.object <- gam(y ~ s(x,6) + z,data=gam.data)
plot(gam.object,se=TRUE)
data(gam.newdata)
preplot(gam.object,newdata=gam.newdata)
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
\name{predict.gam}
\alias{predict.gam}
\title{Predict method for GAM fits}
\description{Obtains predictions and optionally estimates standard errors of
     those predictions from a fitted generalized additive model object.}
\usage{
predict.gam(object, newdata, type, dispersion, se.fit = FALSE,na.action, terms,\dots)
}
\arguments{
\item{object}{
a fitted \code{gam} object, or one of its inheritants, such as a \code{glm} or \code{lm} object.
}
\item{newdata}{
a data frame containing the values at which predictions are required.
This argument can be missing, in which case predictions are made at the same values used to compute the object. 
Only those predictors, referred to in the right side of the formula in object need be present by name in \code{newdata}.
}
\item{type}{
type of predictions, with choices \code{"link"} (the default), \code{"response"}, or \code{"terms"}.
The default produces predictions on the scale of the additive predictors, and with \code{newdata} missing, \code{predict} is simply an extractor function for this component of a \code{gam} object. If \code{"response"} is selected, the predictions are on the scale of the response, and are monotone transformations of the additive predictors, using the inverse link function. If \code{type="terms"} is selected, a matrix of predictions is produced, one column for each term in the model.
}
\item{se.fit}{
if \code{TRUE}, pointwise standard errors are computed along with the predictions.
}
\item{dispersion}{the dispersion of the GLM fit to be assumed in computing
          the standard errors.  If omitted, that returned by 'summary'
          applied to the object is used}
\item{terms}{
if \code{type="terms"}, the \code{terms=} argument can be used to specify which terms should be included; the default is \code{labels(object)}. 
}
\item{na.action}{ function determining what should be done with missing values
  in 'newdata'.  The default is to predict 'NA'.}
\item{\dots}{}
}
\value{
a vector or matrix of predictions, or a list consisting of the predictions and their standard errors if \code{se.fit = TRUE}. 
If \code{type="terms"}, a matrix of fitted terms is produced, with one column for each term in the model (or subset of these if the \code{terms=} argument is used). There is no column for the intercept, if present in the model, and each of the terms is centered so that their average over the original data is zero. 
The matrix of fitted terms has a \code{"constant"} attribute which, when added to the sum of these centered terms, gives the additive predictor.
See the documentation of \code{predict} for more details on the components returned.

When \code{newdata} are supplied, \code{predict.gam} simply invokes
inheritance and gets \code{predict.glm} to produce the parametric part
of the predictions. For each nonparametric term, \code{predict.gam} reconstructs the
partial residuals and weights from the final iteration of the local
scoring algorithm. The appropriate smoother is called for each term,
with the appropriate \code{xeval} argument (see \code{\link{s}} or
\code{\link{lo}}), and the prediction for that term is produced.

The standard errors are based on an approximation given in Hastie
(1992). Currently \code{predict.gam} does not produce standard errors for
predictions at \code{newdata}.

Warning: naive use of the generic
\code{predict} can produce incorrect predictions when the \code{newdata}
argument is used, if the formula in \code{object} involves
transformations such as \code{sqrt(Age - min(Age))}. 
}
\seealso{\code{\link{predict.glm}},
\code{\link{fitted}}, \code{\link{expand.grid}}
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992). This version of \code{predict.gam} is adapted from the S
  version to match the corresponding predict methods for \code{glm} and
  \code{lm} objects in R. The \code{safe.predict.gam} function in S is
  no longer required, primarily because a safe prediction method is in
  place for functions like \code{ns}, \code{bs}, and \code{poly}.
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.

  Venables, W. N. and Ripley, B. D. (2002)
  \emph{Modern Applied Statistics with S.}
  New York: Springer.
}
\examples{
data(gam.data)
gam.object <- gam(y ~ s(x,6) + z, data=gam.data)
predict(gam.object) # extract the additive predictors
data(gam.newdata)
predict(gam.object, gam.newdata, type="terms") 
}

\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}

\eof
\name{s}
\alias{s}
\alias{gam.s}
\title{Specify a Smoothing Spline Fit in a GAM Formula}
\description{A symbolic wrapper to indicate a smooth term in a formala
 argument to gam}
\usage{
s(x, df=4, spar=1)
gam.s(x, y, w, df, spar, xeval)
}
\arguments{
\item{x}{
the univariate predictor, or expression, that evaluates to a numeric vector.
}

\item{df}{ the target equivalent degrees of freedom, used as a smoothing
parameter. The real smoothing parameter (\code{spar} below) is found
such that \code{df=tr(S)-1}, where \code{S} is the implicit smoother
matrix. Values for \code{df} should be greater than \code{1}, with
\code{df=1} implying a linear fit. If both \code{df} and \code{spar} are
supplied, the former takes precedence.}
\item{spar}{
can be used as smoothing parameter, with values typically in
\code{(0,1]}. See \code{\link{smooth.spline}} for more details.}

\item{y}{a response variable passed to \code{gam.s} during backfitting}
\item{w}{weights}
\item{xeval}{If this argument is present, then \code{gam.s} produces a
  prediction at \code{xeval}.}
}
\value{
  
\code{s} returns the vector \code{x}, endowed with a number of
attributes. The vector itself is used in the construction of the model
matrix, while the attributes are needed for the backfitting algorithms
\code{all.wam} (weighted additive model) or \code{s.wam} (currently not
implemented). Since smoothing splines reproduces linear fits, the linear
part will be efficiently computed with the other parametric linear parts
of the model.

Note that \code{s} itself does no smoothing; it simply sets things up
for \code{gam}.

One important attribute is named \code{call}. For example, \code{s(x)}
has a call component
\code{gam.s(data[["s(x)"]], z, w, spar = 1, df = 4)}.
This is an expression that gets evaluated repeatedly in \code{all.wam}
(the backfitting algorithm).

\code{gam.s} returns an object with components
\item{residuals}{The residuals from the smooth fit. Note that the
  smoother removes the parametric part of the fit (using a linear fit
  in \code{x}), so these residual represent the
  nonlinear part of the fit.}
\item{nl.df}{the nonlinear degrees of freedom}
\item{var}{the pointwise variance for the nonlinear fit}

When \code{gam.s} is evaluated with an \code{xeval} argument, it returns a
vector of predictions.
}
\seealso{
\code{\link{lo}}, \code{\link{smooth.spline}}, \code{\link{bs}}, \code{\link{ns}}, \code{\link{poly}}
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992).
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.
}
\examples{
# fit Start using a smoothing spline with 4 df.
y ~ Age + s(Start, 4)
# fit log(Start) using a smoothing spline with 5 df.
y ~ Age + s(log(Start), df=5)
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
\name{step.gam}
\alias{step.gam}
\title{Stepwise model builder for GAM}
\description{Builds a GAM model in a step-wise fashion. For each "term"
  there is an ordered list of alternatives, and the function traverses
  these in a greedy fashion}
\usage{
step.gam(object, scope, scale, direction, trace = TRUE, keep, steps, \ldots)
}
\arguments{
\item{object}{
An object of class \code{gam} or any of it's inheritants.
}
\item{scope}{
defines the range of models examined in the step-wise search. It is a list of formulas, with each formula corresponding to a term in the model. Each of these formulas specifies a "regimen" of candidate forms in which the particular term may enter the model. For example, a term formula might be 
~1+ Income + log(Income) + s(Income)
This means that \code{Income} could either appear not at all,  linearly, linearly in its logarithm, or as a smooth function estimated nonparametrically. A \code{1} in the formula allows the additional option of leaving the term out of the model entirely. 
Every term in the model is described by such a term formula, and the final model is built up by selecting a component from each formula. 


The supplied model \code{object} is used as the starting model, and hence there is the requirement that one term from each of the term formulas be present in \code{formula(object)}. This also implies that any terms in \code{formula(object)} not contained in any of the term formulas will be 
to be present in every model considered.
}
\item{scale}{
an optional argument used in the definition of the AIC statistic used to evaluate models for selection. By default, the scaled Chi-squared statistic for the initial model is used, but if forward selection is to be performed, this is not necessarily a sound choice. 
}
\item{direction}{
The mode of step-wise search, can be one of \code{"both"}, \code{"backward"}, or \code{"forward"}, with a default of \code{"both"}. If \code{scope} is missing, the default for \code{direction} is "backward". 
}
\item{trace}{
If \code{TRUE}, information is printed during the running of \code{step.gam()}. This is an encouraging choice in general, since \code{step.gam()} can take some time to compute either for large models or when called with an an extensive \code{scope=} argument. A simple one line model summary is printed for each model visited in the search, and the selected model is noted at each step.
}
\item{keep}{
A filter function whose input is a fitted \code{gam} object and the associated "AIC" statistic, and whose output is arbitrary. Typically \code{keep()} will select a subset of the components of the object and return them. The default is not to keep anything.
}
\item{steps}{
The maximum number of steps to be considered. The default is 1000 (essentially as
many as required). It is typically used to stop the process early.
}
\item{\dots}{Additional arguments to be passed on to \code{keep}}
}
\value{
The step-wise-selected model is returned, with up to two additional components.
There is an \code{"anova"} component corresponding to the steps taken in the search, as well as a \code{"keep"} component if the \code{keep=} argument was supplied in the call. 


We describe the most general setup, when \code{direction = "both"}.
At any stage there is a current model comprising a single term from each of the term formulas supplied in the \code{scope=} argument.
A series of models is fitted, each corrresponding to a formula obtained by moving each of the terms one step up or down in its regimen, relative to the formula of the current model.
If the current value for any term is at either of the extreme ends of its regimen, only one rather than two steps can be considered.
So if there are \code{p} term formulas, at most \code{2*p - 1}  models are considered.
A record is kept of all the models ever visited (hence the \code{-1} above), to avoid repetition.
Once each of these models has been fit, the "best" model
in terms of the AIC statistic is selected and defines the step.
The entire process is repeated until either the maximum number of steps has been used, or until the AIC criterion can not be decreased by any of the eligible steps.
}
\seealso{
\code{\link{step}},\code{\link{glm}}, \code{\link{gam}}, \code{\link{drop1}}, \code{\link{add1}}, \code{\link{anova.gam}}
}
\author{
  Written by Trevor Hastie, following closely the design in the
  "Generalized Additive Models" chapter (Hastie, 1992) in Chambers and
  Hastie (1992). 
 }
\references{
  Hastie, T. J. (1992)
  \emph{Generalized additive models.}
  Chapter 7 of \emph{Statistical Models in S}
  eds J. M. Chambers and T. J. Hastie, Wadsworth \& Brooks/Cole.

  Hastie, T. and Tibshirani, R. (1990)
  \emph{Generalized Additive Models.}
  London: Chapman and Hall.
}
\examples{
data(gam.data)
gam.object <- gam(y~x+z, data=gam.data)
step.object <-step.gam(gam.object, scope=list("x"=~1+x+s(x,4)+s(x,6)+s(x,12),"z"=~1+z+s(z,4)))
}
\keyword{models}
\keyword{regression}
\keyword{nonparametric}
\keywords{smooth}


\eof
